wPortalsDB = {
	autoRestock = false,
	tele = 20,
	port = 20,
	powder = 100
}

---------------------------------------------------
-- Constants (not really, just don't change them!)
---------------------------------------------------

local NUM_TELEPORTS = 9;
local NUM_PORTALS = 9;
local WP = "|cffe1a500w|cff69ccf0Portals|cffffff00";

local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4,}
}

local addOnVersion = GetAddOnMetadata("wPortals", "Version")

---------------------------------------------------
-- Player info and appropriate actions 
---------------------------------------------------

local _, class = UnitClass("player")
if (class~="MAGE") then
	DisableAddOn("wPortals")
	print(string.format("%s has been disabled for this character since it's not a mage.",WP));
end
local faction, _ = UnitFactionGroup("player")
local teleRunes, portRunes, arcanePowder = 0,0,0

---------------------------------------------------
-- Location table {teleID, portID}
---------------------------------------------------

local Locs = {
	Alliance = {
		{88342, 88345}, -- Tol Barad
		{53140, 53142}, -- Dalaran
		{33690, 33691}, -- Shattrath
		{3561, 10059}, -- Stormwind
		{3562, 11416}, -- Ironforge
		{3565, 11419}, -- Darnassus
		{32271, 32266}, -- Exodar
		{49359, 49360}, -- Theramore
		{132621, 132620} -- Vale of Eternal Blossoms
	},
	Horde = {
		{88344, 88346}, -- Tol Barad
		{53140, 53142}, -- Dalaran
		{35715, 35717}, -- Shattrath
		{3567, 11417}, -- Orgrimmar
		{3563, 11418}, -- Undercity
		{3566, 11420}, -- Thunder Bluff
		{32272, 32267}, -- Silvermoon
		{49358, 49361}, -- Stonard
		{132627, 132626} -- Vale of Eternal Blossoms
	},
}

---------------------------------------------------
-- Main frame and it's jungle jam!
---------------------------------------------------

local main = CreateFrame("Frame", "wPortalsMainFrame", UIParent)

local teleButtons = {}
local portButtons = {}
for i = 1,NUM_TELEPORTS do
	local f = CreateFrame("Button",nil,main,"SecureActionButtonTemplate")
	f:SetSize(25,25)
	local g = CreateFrame("Button",nil,main,"SecureActionButtonTemplate")
	g:SetSize(25,25)
	if (i==1) then
		f:SetPoint("TOPLEFT",main,10,-10)
		g:SetPoint("TOP",f,"BOTTOM",0,-10)
	else
		f:SetPoint("LEFT",teleButtons[i-1],"RIGHT",5,0)
		g:SetPoint("LEFT",portButtons[i-1],"RIGHT",5,0)
	end
	table.insert(teleButtons,f)
	table.insert(portButtons,g)
end

-- Set functions of teleport buttons
local function teleUpdate()
	for i = 1,NUM_TELEPORTS do
		local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(Locs[faction][i][1])
		teleButtons[i]:SetNormalTexture(icon)
		if (IsSpellKnown(Locs[faction][i][1])==true) then
			teleButtons[i]:SetAttribute("type1", "spell")
			teleButtons[i]:SetAttribute("spell", Locs[faction][i][1])
			teleButtons[i]:Enable(true)
			teleButtons[i]:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT"); GameTooltip:ClearLines(); GameTooltip:AddLine(name); GameTooltip:Show() end)
		else
			teleButtons[i]:Enable(false)
			teleButtons[i]:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT"); GameTooltip:ClearLines(); GameTooltip:AddLine(name); GameTooltip:AddLine("You do not know this spell yet!",1,0,0,1) GameTooltip:Show() end)
		end
		teleButtons[i]:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	end
end

-- Set functions of portal buttons
local function portUpdate()
	for i = 1,NUM_PORTALS do
		local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(Locs[faction][i][2])
		portButtons[i]:SetNormalTexture(icon)
		if (IsSpellKnown(Locs[faction][i][2])==true) then
			portButtons[i]:SetAttribute("type1", "spell")
			portButtons[i]:SetAttribute("spell", Locs[faction][i][2])
			portButtons[i]:Enable(true)
			portButtons[i]:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT"); GameTooltip:ClearLines(); GameTooltip:AddLine(name); GameTooltip:Show() end)
		else
			portButtons[i]:Enable(false)
			portButtons[i]:SetScript("OnEnter", function(self) GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT"); GameTooltip:ClearLines(); GameTooltip:AddLine(name); GameTooltip:AddLine("You do not know this spell yet!",1,0,0,1) GameTooltip:Show() end)
		end
		portButtons[i]:SetScript("OnLeave", function(self) GameTooltip:Hide() end)
	end
end

main:SetBackdrop(backdrop)
main:SetBackdropColor(0,0,0,0.9)
main:SetSize(285,115)
main:SetPoint("CENTER",UIParent)
main:EnableMouse(true)
main:SetMovable(true)
main:SetUserPlaced(true)
main:SetScript("OnMouseDown", function(self, button) if button=="LeftButton" then self:StartMoving() end end)
main:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
main:Hide()

-- Old tracker function, left if reagents ever come back :(
local numTrackers = 0;
local function createTracker(name,tex,tooltip)
	local f = CreateFrame("Button",(name.."Tracker"),main)
	f:SetSize(25,25)
	f:SetNormalTexture(tex)
	f:SetScript("OnEnter",function(self) GameTooltip:SetOwner(self,"ANCHOR_TOPLEFT") GameTooltip:ClearLines() GameTooltip:AddLine(tooltip) GameTooltip:Show() end)
	f:SetScript("OnLeave",function(self) GameTooltip:Hide() end)
	f:SetPoint("BOTTOMLEFT", main, "BOTTOMLEFT", numTrackers*30+10, 10);
	--f:SetPoint(anch[1],anch[2],anch[3],anch[4],anch[5])
	local g = f:CreateFontString((name.."Text"),"OVERLAY","ChatFontSmall")
	g:SetPoint("BOTTOMRIGHT",f,-2,2)
	g:SetFont("fonts\\frizqt__.ttf",11,"OUTLINE")
	g:SetText("0");
	numTrackers = numTrackers + 1;
	return f, g	
end

main:SetScript("OnShow", function(self)
	faction, _ = UnitFactionGroup("player") -- Just incase the player changes faction?
	
	-- Make sure tele/port buttons are up to date
	teleUpdate()
	portUpdate()
end)

local versionText = main:CreateFontString("wPortalsVersionText","OVERLAY","ChatFontNormal");
versionText:SetPoint("BOTTOMLEFT",main,10,10);
versionText:SetText(string.format("%s|cffffffff ver. %.2f",WP,addOnVersion));
versionText:SetFont("fonts\\frizqt__.ttf",15,"OUTLINE");

local closeButton = CreateFrame("Button",nil, main,"OptionsButtonTemplate")
closeButton:SetPoint("BOTTOMRIGHT", main, "BOTTOMRIGHT", -10, 10)
closeButton:SetScript("OnClick", function(self) main:Hide() end)
closeButton:SetSize(60,20)
closeButton:SetText("Close")


---------------------------------------------------
-- Slash commands
---------------------------------------------------

SLASH_WPORTALS1 = '/port'
SLASH_WPORTALS2 = '/ports'
SLASH_WPORTALS3 = '/wport'
SLASH_WPORTALS4 = '/wportals'
function SlashCmdList.WPORTALS(msg,editbox)
	main:Show()
end

---------------------------------------------------
-- LDB button
---------------------------------------------------

wPortalsBroker = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_wPortals", {
	type = "launcher",
	icon = "Interface\\Icons\\Spell_Arcane_PortalDalaran",
	label = "wPortals",
	OnClick = function(_, button)
		if button == "LeftButton" then
			main:Show()
		end
	end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine(WP)
		tooltip:AddLine("|cff69ccf0Left Click|cffffd200 to show the portals frame.")
	end,
})
